CREATE TABLE IF NOT EXISTS `zt_boardlayer` (
    `id` bigint unsigned NOT NULL AUTO_INCREMENT,
    `canvas` bigint unsigned NOT NULL COMMENT '画板id',
    `subID` varchar(64) CHARACTER SET utf8mb4 COLLATE utf8mb4_unicode_ci NOT NULL COMMENT '组件id',
    `type` varchar(16) CHARACTER SET utf8mb4 COLLATE utf8mb4_unicode_ci NOT NULL COMMENT '类型',
    `data` text COLLATE utf8mb4_unicode_ci NOT NULL COMMENT '便签内容',
    `cards` text COLLATE utf8mb4_unicode_ci COMMENT '卡片',
    `createdDate` datetime DEFAULT NULL,
    `createdBy` varchar(30) NOT NULL DEFAULT '',
    `editedDate` datetime DEFAULT NULL,
    `endCards` text COLLATE utf8mb4_unicode_ci COMMENT '卡片',
    `sort` int NOT NULL DEFAULT '1' COMMENT '排序',
    PRIMARY KEY (`id`),
    UNIQUE KEY `layer_sub_id_unique` (`subId`),
    KEY `layer_canvas_id_index` (`canvas`,`editedDate`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

CREATE TABLE IF NOT EXISTS `zt_boardcanvas` (
    `id` bigint unsigned NOT NULL AUTO_INCREMENT,
    `name` varchar(255) CHARACTER SET utf8mb4 COLLATE utf8mb4_unicode_ci DEFAULT NULL COMMENT '画板名',
    `cover` varchar(255) CHARACTER SET utf8mb4 COLLATE utf8mb4_unicode_ci DEFAULT NULL COMMENT '封面',
    `status` varchar(30) NOT NULL DEFAULT 'unclosed',
    `actionsStatus` tinyint NOT NULL DEFAULT '1' COMMENT '画板状态',
    `players` text COLLATE utf8mb4_unicode_ci COMMENT '参与人',
    `usersCount` int unsigned NOT NULL DEFAULT '0' COMMENT '白板人数',
    `templateData` text COLLATE utf8mb4_unicode_ci COMMENT '模版数据',
    `zoom` double(3,2) NOT NULL DEFAULT '1.00' COMMENT '比例',
    `coordinates` text COLLATE utf8mb4_unicode_ci COMMENT '坐标',
    `template` bigint NOT NULL DEFAULT '0' COMMENT '使用的模板',
    `acl` varchar(10) NOT NULL DEFAULT 'open',
    `groups` varchar(255) NOT NULL DEFAULT '',
    `users` text,
    `collector` text,
    `collaborator` text,
    `lib` bigint unsigned NOT NULL DEFAULT '0',
    `type` varchar(30) NOT NULL DEFAULT '',
    `owner` char(30) NOT NULL DEFAULT '',
    `createdDate` datetime DEFAULT NULL,
    `createdBy` char(30) NOT NULL DEFAULT '',
    `editedDate` datetime DEFAULT NULL,
    `deleted` tinyint NOT NULL DEFAULT '0' COMMENT '是否删除',
    `locked` tinyint unsigned NOT NULL DEFAULT '0' COMMENT '是否被锁定',
    PRIMARY KEY (`id`),
    KEY `index_0` (`createdBy`,`createdDate`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

CREATE TABLE IF NOT EXISTS `zt_boardtemplate` (
    `id` bigint unsigned NOT NULL AUTO_INCREMENT,
    `canvas` bigint unsigned NOT NULL COMMENT '画板id',
    `name` varchar(255) CHARACTER SET utf8mb4 COLLATE utf8mb4_unicode_ci DEFAULT NULL COMMENT '模板名称',
    `cover` varchar(255) CHARACTER SET utf8mb4 COLLATE utf8mb4_unicode_ci DEFAULT NULL COMMENT '模板封面',
    `introduction` varchar(255) CHARACTER SET utf8mb4 COLLATE utf8mb4_unicode_ci DEFAULT NULL COMMENT '模板简介',
    `status` tinyint NOT NULL DEFAULT '1' COMMENT '是否显示',
    `sort` int DEFAULT NULL COMMENT '排序',
    `createdDate` datetime NULL DEFAULT NULL,
    `createdBy` char(30) NOT NULL DEFAULT '',
    `editedDate` datetime NULL DEFAULT NULL,
    `usedNum` bigint unsigned NOT NULL DEFAULT '0' COMMENT '复制数',
    PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

CREATE TABLE IF NOT EXISTS `zt_boardlib` (
    `id` bigint unsigned NOT NULL AUTO_INCREMENT,
    `type` varchar(30) NOT NULL DEFAULT '',
    `vision` varchar(10) NOT NULL DEFAULT 'rnd',
    `product` mediumint unsigned NOT NULL DEFAULT '0',
    `project` mediumint unsigned NOT NULL DEFAULT '0',
    `execution` mediumint unsigned NOT NULL DEFAULT '0',
    `name` varchar(60) NOT NULL DEFAULT '',
    `acl` varchar(10) NOT NULL DEFAULT 'open',
    `groups` varchar(255) NOT NULL DEFAULT '',
    `users` text,
    `collector` text,
    `desc` mediumtext,
    `order` tinyint unsigned NOT NULL DEFAULT '0',
    `createdBy` varchar(30) NOT NULL DEFAULT '',
    `createdDate` datetime DEFAULT NULL,
    `deleted` enum('0','1') NOT NULL DEFAULT '0',
    PRIMARY KEY (`id`),
    KEY `product` (`product`),
    KEY `execution` (`execution`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

CREATE TABLE IF NOT EXISTS `zt_boardaction` (
    `id` bigint unsigned NOT NULL AUTO_INCREMENT,
    `canvas` bigint unsigned NOT NULL DEFAULT '0',
    `action` varchar(80) NOT NULL DEFAULT '',
    `actor` char(30) NOT NULL DEFAULT '',
    `extra` text,
    `date` datetime DEFAULT NULL,
    PRIMARY KEY (`id`),
    KEY `canvas` (`canvas`),
    KEY `actor` (`actor`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;
