<?php
/**
 * 获取创建需求的表单字段。
 * Get form fields for create
 *
 * @param  int       $productID
 * @param  string    $branch
 * @param  int       $objectID
 * @param  object    $initStory
 * @param  string    $storyType
 * @access protected
 * @return array
 */
protected function getFormFieldsForCreate($productID, $branch, $objectID, $initStory, $storyType = 'story', $extra = '')
{
    $account = $this->app->user->account;
    $fields  = $this->config->story->form->create;
    /* 准备数据。*/
    list($products, $branches) = $this->getProductsAndBranchesForCreate($productID, $objectID);
    if($objectID)
    {
        $branch    = is_array($branches) && !empty($branches) ? (string)key($branches) : $branch;
        $productID = (!empty($productID) && isset($products[$productID])) ? $productID : key($products);
    }
    $branch = str_contains($branch, ',') ? current(explode(',', $branch)) : $branch;
    $product        = $this->product->getByID($productID);
    $users          = $this->user->getPairs('pdfirst|noclosed|nodeleted');
    $stories        = $this->story->getParentStoryPairs($productID, '', $storyType);
    $grades         = $this->story->getGradePairs($storyType);
    $plans          = $this->loadModel('productplan')->getPairs($productID, $branch == 0 ? '' : $branch, 'unexpired|noclosed', true);
    $plans          = array_map(function($planName){return str_replace(FUTURE_TIME, $this->lang->story->undetermined, $planName);}, $plans);
    $forceReview    = $this->story->checkForceReview($storyType);
    $needReview     = ($account == $product->PO || $objectID > 0 || $this->config->{$storyType}->needReview == 0 || !$forceReview);
    $reviewers      = $this->story->getProductReviewers($productID);
    $requiredFields = $this->config->{$storyType}->create->requiredFields;
    /* 追加字段的name、title属性，展开user数据。 */
    foreach($fields as $field => $attr)
    {
        if(isset($attr['options']) and $attr['options'] == 'users') $fields[$field]['options'] = $users;
        if(!isset($fields[$field]['name']))       $fields[$field]['name']     = $field;
        if(!isset($fields[$field]['title']))      $fields[$field]['title']    = zget($this->lang->story, $field);
        if(str_contains($requiredFields, $field)) $fields[$field]['required'] = true;
    }

    if(!empty($extra)) 
    {
        $extraData = str_replace(array(',', ' '), array('&', ''), $extra);
        parse_str($extraData, $output);
        $fromType = isset($output['fromType']) ? $output['fromType'] : '';
        if(!empty($fromType) && $fromType === 'board') $products = $this->loadModel('product')->getPairs();
    }
    /* 设置下拉菜单内容。 */
    $fields['product']['options']  = $products;
    $fields['branch']['options']   = $branches;
    $fields['branches']['options'] = $branches;
    $fields['plan']['options']     = $plans;
    $fields['plans']['options']    = $plans;
    $fields['grade']['options']    = $grades;
    $fields['grade']['default']    = current($grades);
    $fields['reviewer']['options'] = $reviewers;
    $fields['parent']['options']   = array_filter($stories);
    /* 设置默认值。 */
    foreach($initStory as $field => $defaultValue)
    {
        if(isset($fields[$field])) $fields[$field]['default'] = $defaultValue;
    }
    if(empty($fields['product']['default']))  $fields['product']['default']  = $productID;
    if(empty($fields['branch']['default']))   $fields['branch']['default']   = (int)$branch;
    if(empty($fields['branches']['default'])) $fields['branches']['default'] = (int)$branch;
    if(empty($fields['plans']['default']))    $fields['plans']['default']    = zget($initStory, 'plan', 0);
    if(empty($needReview)) $fields['reviewer']['default']  = $product->PO;
    if($forceReview)       $fields['reviewer']['required'] = true;
    /* 删除不需要的字段。 */
    if(empty($branches)) unset($fields['branch'], $fields['branches'], $fields['modules'], $fields['plans']);
    if($this->config->vision == 'or') unset($fields['plan']);
    $this->view->productID   = $productID;
    $this->view->product     = $product;
    $this->view->branch      = $branch;
    $this->view->branches    = $branches;
    $this->view->objectID    = $objectID;
    $this->view->forceReview = $forceReview;
    $this->view->needReview  = $needReview;
    $this->view->gradeRule   = $this->config->{$storyType}->gradeRule;

    return $fields;
}
