<?php
namespace zin;

class boardEditor extends wg
{
    protected static array $defineProps = array
    (
        'id'       => '?string',
        'canvasID' => '?string',
        'canvas'   => '?object',
        'actions'  => '?array',
        'config'   => '?array'
    );

    protected function getConfig()
    {
        global $app, $config, $lang;
        $webRoot   = $app->getWebRoot();
        list($canvasID, $canvas) = $this->prop(array('canvasID', 'canvas'));

        $ui = array();
        $ui['openUrl']      = jsRaw('(url) => openUrl(url)');
        $ui['enableStatistics'] = false;
        $ui['afterQuit']    = jsRaw('() => goBack("APP", "' . createLink('board', 'teamBoard') . '")');
        $ui['avatarRender'] = jsRaw('(props) => ({html: zui.Avatar.renderHTML({circle: true, text: props.name, size: props.size, code: props.id, src: props.img}), class: "center rounded-full"})');

        $urlMap = array();
        $urlMap['openBoard']    = createLink('board', 'open');
        $urlMap['getUserInfo']  = createLink('board', 'ajaxGetUser', "canvasID=$canvasID");
        $urlMap['bindBoard']    = createLink('board', 'bind', "id=$canvasID");
        $urlMap['setTimer']     = createLink('board', 'ajaxSetTimer', "type={type}&canvasID=$canvasID");
        $urlMap['setCover']     = createLink('board', 'ajaxSetCover');
        $urlMap['setName']      = createLink('board', 'ajaxSetName', "canvasID=$canvasID");
        $urlMap['collectBoard'] = createLink('board', 'ajaxCollect', "canvasID=$canvasID");
        $urlMap['autoUpdate']   = createLink('board', 'autoUpdate');
        $urlMap['setStatus']    = createLink('board', 'ajaxSetStatus');
        $urlMap['quitBoard']    = createLink('board', 'quit');
        $urlMap['setViewport']  = createLink('board', 'ajaxSetViewport', "canvasID=$canvasID");
        $urlMap['updateLayer']  = createLink('board', 'ajaxUpdateLayer', "canvasID=$canvasID");
        $urlMap['updateStory']  = createLink('board', 'ajaxUpdateStory', "canvasID=$canvasID");
        $urlMap['getBlocks']    = createLink('board', 'ajaxGetBlocks');
        $urlMap['uploadImage']  = createLink('file', 'ajaxUpload', "uid=board$canvasID&objectType=board&objectID=$canvasID&extra&field=image&api=1");

        $api = array();
        $api['basePath']   = '/';
        $api['urlMap']     = $urlMap;
        $api['version']    = 1;
        $api['ws']         = $config->board->ws;

        $ai = array('enabled' => false);

        $boardActions = array();
        $boardActions[] = array('key' => 'members');
        $boardActions[] = array('type' => 'divider');
        $actionsSetting = $this->prop('actions');
        if($actionsSetting) $boardActions = array_merge($boardActions, array_values($actionsSetting), array(array('type' => 'divider')));
        $boardActions[] = array('key' => 'help');

        $defaultNoteSize = array();
        $defaultNoteSize['mode_media']  = array('width' => 280, 'height' => 357, 'textSize' => 24);
        $defaultNoteSize['mode_pen']    = array('width' => 280, 'height' => 252, 'textSize' => 24);
        $defaultNoteSize['horizontal']  = array('width' => 320, 'height' => 70,  'textSize' => 34);
        $defaultNoteSize['circle']      = array('width' => 200, 'height' => 200, 'textSize' => 34);
        $defaultNoteSize['diamond']     = array('width' => 200, 'height' => 160, 'textSize' => 34);
        $defaultNoteSize['bubble']      = array('width' => 320, 'height' => 70,  'textSize' => 34);
        $defaultNoteSize['default']     = array('width' => 200, 'height' => 200, 'textSize' => 34);

        $convertActions = $lang->board->actions['convert'];
        foreach($lang->board->actions['convert'] as $key => $action) $convertActions[$key] = array('key' => $key, 'text' => $action);

        $enableER = isset($config->enableER) && empty($config->enableER);
        $enableUR = isset($config->URAndSR) && empty($config->URAndSR);

        $createER      = !$enableER && hasPriv('epic', 'create');
        $createUR      = !$enableUR && hasPriv('requirement', 'create');
        $createSR      = hasPriv('story', 'create');
        $createDR      = hasPriv('demand', 'create');
        $batchCreateER = !$enableER && hasPriv('epic', 'batchCreate');
        $batchCreateUR = !$enableUR && hasPriv('requirement', 'batchCreate');
        $batchCreateSR = hasPriv('story', 'batchCreate');
        $batchCreateDR = hasPriv('demand', 'batchCreate');

        if($config->vision === 'or')
        {
            array_pop($convertActions);
        }
        elseif($config->vision === 'lite')
        {
            $convertActions = [];
        }
        else
        {
            array_shift($convertActions);
        }

        $board = array();
        $board['customLogoBtn']          = array('label' => $lang->goback, 'class' => 'open-url', 'data-command' => '!canvas.quit?confirm=true', 'iconClass' => 'icon icon-back');
        $board['noSaveTime']             = true;
        $board['disableImportCard']      = true;
        $board['disableAddMediaNode']    = true;
        $board['addBlockLabel']          = $lang->board->editor->addBlock;
        $board['disableBoardDataTable']  = true;
        $board['disableInvite']          = true;
        $board['hideNameInUserBar']      = true;
        $board['noteColors']             = $lang->board->editor->noteColors;
        $board['noteTextSizeList']       = array(12, 18, 24, 34, 44, 48, 88, 180);
        $board['vipNoteColors']          = false;
        $board['disableGuide']           = true;
        $board['boardActions']           = $boardActions;
        $board['useLocalHelpImage']      = true;
        $board['defaultNoteSize']        = $defaultNoteSize;
        $board['readonlyMessage']        = $canvas->deleted == 1 ? $lang->board->editor->deleteMessage : $lang->board->editor->readonlyMessage;
        $board['readonlyAction']         = ($actionsSetting && isset($actionsSetting['activate'])) ? $actionsSetting['activate'] : array('class' => 'hidden');
        $board['uploadImageAsTemplet']   = true;
        $board['isDeleted']              = $canvas->deleted == 1;
        $board['convertActions']         = $convertActions;
        $board['convertMessage']         = $lang->board->hint->converted;
        $board['convertGroupMessage']    = $lang->board->hint->convertedCroup;
        $board['batchCreateER']          = $batchCreateER;
        $board['batchCreateUR']          = $batchCreateUR;
        $board['batchCreateSR']          = $batchCreateSR;
        $board['batchCreateDR']          = $batchCreateDR;
        $board['createER']               = $createER;
        $board['createUR']               = $createUR;
        $board['createSR']               = $createSR;
        $board['createDR']               = $createDR;

        $cdn = array();
        $cdn['imgCdn']           = $webRoot;
        $cdn['useLocalResource'] = true;

        $env = array();
        $env['disableVip']       = true;
        $env['isPrivate']        = true;
        $env['basePath']         = $webRoot;
        $env['publicPath']       = (isset($config->boardLibPath) ? $config->boardLibPath : $config->zin->zuiPath) . 'zboard/';
        $env['useLocalResource'] = true;

        $boardConfig     = compact('ui', 'api', 'ai', 'board', 'cdn', 'env');
        $userBoardConfig = $this->prop('config');
        if($userBoardConfig) $boardConfig = array_replace_recursive($boardConfig, $userBoardConfig);
        return $boardConfig;
    }

    protected function build()
    {
        global $config, $app;
        $boardConfig = js::decodeRaw(json_encode($this->getConfig()));
        $crossorigin = preg_match('/^https?\:\/\//', $config->zin->zuiPath) ? 'anonymous' : null;
        $id = $this->prop('id');
        if(!$id) $id = 'boardEditor';
        $libPath = isset($config->boardLibPath) ? $config->boardLibPath : $config->zin->zuiPath;
        return array
        (
            h::importCss($libPath . 'zboard/zboard.css', set::crossorigin($crossorigin)),
            div(setID($id)),
            zui::zboard
            (
                set::id($this->prop('canvasID')),
                set::lang($app->getClientLang()),
                set($this->getRestProps()),
                set::_to("#$id"),
                set::_call(<<<JS
                (function(name,selector,options)
                {
                    $.getLib('zboard/zboard.umd.cjs', {root: "$libPath"}, () =>
                    {
                        const BoardEditor = zboard.BoardEditor;
                        const zuiName = 'boardEditor';
                        $.extend(BoardEditor,
                        {
                            NAME: 'BoardEditor',
                            ZUI: zuiName,
                            KEY: 'zui.' + zuiName,
                            get: () => $(selector).data('zui.' + zuiName)
                        });
                        $.extend(zui, {BoardEditor: BoardEditor});
                        if(zui.registerComponent) zui.registerComponent(BoardEditor);
                        options = $.extend({config: $boardConfig}, options);
                        if(window.config && window.config.debug) console.log('> boardEditor.create', options);
                        zui.create(zuiName, selector, options);
                    });
                })
                JS)
            )
        );
    }
}
