const objectSpaceY = 8;
const objectSpaceX = 40;
const objectHeight = 96;
const objectWidth = 366;
let importResolver;

window.goBack = parent.goBack;

window.updateBoardHistory = function()
{
    const history = $('#boardHistory .history-panel').zui();
    if(history) history.$.load();
};

window.toggleBoardHistory = function(toggle)
{
    $('body').toggleClass('show-history', toggle);

    if($('body').hasClass('show-history'))
    {
        updateBoardHistory();
        zboard.cmd.executeCommandLine('canvas.toggleSidebar');
    }
};

function getImportData(item, props)
{
    return props.reduce((data, prop) =>
    {
        data[prop] = item[prop];
        return data;
    }, {});
}

function groupList(list) {
    const map     = new Map();
    const root    = { id: 0, self: null, children: [] };
    const listIDs = list.map(item => item.id);
    list.forEach(item => {
        if(Array.isArray(item.parent)) item.parent = item.parent[item.parent.length - 1];
        const parentId = item.parent && item.parent > 0 ? item.parent : 0;

        let group = map.get(item.id);
        if(group)
        {
            group.self = item;
        }
        else
        {
            group = { id: item.id, self: item, children: [] };
            map.set(item.id, group);
        }

        if(parentId && listIDs.includes(parentId))
        {
            let parentGroup = map.get(parentId);
            if(parentGroup)
            {
                parentGroup.children.push(group);
            }
            else
            {
                parentGroup = { id: parentId, self: null, children: [group] };
                map.set(parentId, parentGroup);
            }
        }
        else
        {
            root.children.push(group);
        }
    });

    return root.children;
}

const objectImportProps =
{
    story: ['id', 'title', 'pri', 'status', 'estimate', 'openedBy', 'openedDate', 'parent', 'type', 'color', 'grade', 'isParent'],
    demand: ['id', 'title', 'pri', 'status', 'estimate', 'createdBy', 'createdDate', 'parent', 'type', 'color', 'grade', 'isParent']
};

window.importObjectToBoard = function(data, options)
{
    if(config.debug) console.log('> importStoryToBoard', data, options);
    if(!data || !importResolver) return;
    const elements = [];
    const x = importResolver.x;
    const objectType = options.objectType || importResolver.objectType;
    const y = importResolver.y;
    const groups = groupList(data.sort((a, b) => b.id - a.id));
    const ids = [];
    const rowHeight = objectHeight + objectSpaceY;
    let   maxIndex = 0;
    const importStory = (item, parentNodeID, xGrade = 0, index = 0) =>
    {
        const id = zui.BoardEditor.createElementId();
        ids.push(id);

        if(index) maxIndex ++;
        const itemX = x + (objectSpaceX + objectWidth) * xGrade;
        const itemY = y + rowHeight * maxIndex;

        elements.push(
        {
            id: id,
            objectType: objectType,
            x: itemX,
            y: itemY,
            width: objectWidth,
            height: objectHeight,
            data: $.extend(getImportData(item, objectImportProps[objectType] || objectImportProps.story), {openedByName: options.userMap[item.openedBy || item.createdBy]})
        });

        if(parentNodeID)
        {
            elements.push(
            {
                type: 'link',
                from: parentNodeID,
                to: id,
                fromSide: 'right',
                toSide: 'left',
                weight: 1,
                color: 'var(--color-secondary-500)',
                shape: 'fold',
                toPoint: 'arrow',
                isLock: true
            });
        }
        return id;
    };

    const importGroups = (groupList, parentNodeID, xGrade = 0, index) =>
    {
        const hasChildren = groupList.children && groupList.children.length;
        if(groupList.self && !parentNodeID) parentNodeID = importStory(groupList.self, 0, 0, index);
        if(hasChildren)
        {
            groupList.children.forEach((item, index) =>
            {
                const childrenID = importStory(item.self, parentNodeID, xGrade + 1, index);
                if(item.children && item.children.length)
                {
                    importGroups(item, childrenID, xGrade + 1, 0);
                }
            });
        }
    }

    groups.forEach((group, index) =>
    {
        importGroups(group, 0, 0, index);
    })

    if(ids.length > 1) elements.push({type: 'group', layers: ids});
    importResolver.resolve(elements);
    updateBoardLayers();
};

window.openImportModal = function(options, board)
{
    importResolver = null;
    zui.Modal.open({url: $.createLink('board', `import${options.objectType}`, 'boardID=' + board.id)});
    return new Promise((resolve) =>
    {
        importResolver = $.extend({resolve}, options);
    });
};

window.renderUserAvatar = function(account, realname)
{
    return zui.Avatar.renderHTML({circle: true, size: 20, src: usersAvatar[account], code: account, text: realname || account});
};

window.onRenderObject = function(object)
{
    const canEditStory = $('#boardHistory').data('canEditStory');
    let data           = object.data;
    let module         = object.objectType;
    if(storyType === 'story' && object.objectType === 'epic') module = 'epic';
    if(storyType === 'demand') module = object.objectType;

    let grade = (data.grade && gradeGroup[data.type] && gradeGroup[data.type][data.grade]) ? gradeGroup[data.type][data.grade].name : langChildrenAB;
    if(module == 'demand' && data.parent && data.parent < 1)  grade = '';
    if(!data.typeName && storyTypeList[module]) data.typeName = storyTypeList[module];
    const html =
        [
            '<div class="row items-center flex-nowrap gap-2">',
            ((data.parent && data.parent > 0) || grade) ? `<div class="label rounded-full size-sm primary-pale justify-start whitespace-nowrap text-clip" title="${grade}">${grade}</div>` : '',
            !canEditStory ? `<div class="pri-${data.pri} flex-none">${data.pri || ''}</div>` : `<div class="cursor-pointer flex items-center story-pri" key="${object.id}" data-story="${data.id}" data-type="${module}" onclick='window.showPriList(this, ${data.id})'><div class="pri-${data.pri} flex-none">${data.pri || ''}</div><i class="icon icon-caret-down text-gray-300"></i></div>`,
            `<div class="flex-auto line-clamp-1 text-clip"><a data-href="${$.createLink(module, 'view', 'storyID=' + data.id)}" class="text-lg story-link" draggable="false"${data.color ? ` style="color:${data.color}"` : ''} title="${data.title}">${data.title}</a></div>`,
            '</div>',
            '<div class="row items-center flex-nowrap gap-2 mt-1">',
            (data.openedBy || data.createdBy) ? renderUserAvatar(data.openedBy || data.createdBy, data.openedByName) : '',
            !canEditStory ? (Number.isNaN(+data.estimate) ? '' : `<div class="text-sm flex-auto"><span class="opacity-50">${langEstimate}</span> ${data.estimate} ${hourUnit}</div>`)
            : (Number.isNaN(+data.estimate) ? '' : (`<div class="text-sm flex-auto"><span class="opacity-50">${langEstimate}</span>` + (data.isParent == '1' ? `<div class="inline-block text-center" style="min-width: 20px;">${data.estimate}</div>${hourUnit}</div>` : `<div class="inline-block border-b border-gray-300"><input class="w-5 text-center estimate-input" data-story-id="${data.id}" value=${data.estimate} data-type="${module}" /></div> ${hourUnit}</div>`))),
            '<div class="flex-auto"></div>',
            Number(data.deleted) ? `<div class="text-danger">${deleted}</div>` : `<div class="status-${data.status}">${storyStatusList[data.status] || data.status}</div>`,
            '</div>',
            data.typeName ? `<span class="label primary-pale size-sm absolute top-px right-px p-2" style="border-radius: 0 var(--radius-sm) 0 var(--radius)">${data.typeName}</span>` : '',
        ].join('');
    return {class: 'canvas relative ring w-24 shadow rounded pt-6 py-5 px-3 col gap-1 justify-evenly w-full', html: html};
};

window.onRenderConvertInfo = function(object, linkStyle, linkWidth)
{
    if(!object?.data) return {html: ''};
    const data   = object.data;
    const module = data.objectType;
    const url    = $.createLink(module, 'view', `${module == 'demand' ? 'demand' : 'story'}ID=${data.id}`);
    const html   = Number(data.deleted) ? '' : `<a style="max-width: ${linkWidth};" class="cursor-pointer flex items-center gap-1 hide-in-silent truncate flex-auto text-right hover:scale-105 transition-all story-link ${linkStyle}" data-href="${url}" title="${data.title}"><i class="icon icon-product"></i>${data.title}</a>`;
    return {html: html};
}

window.updateBoardLayers = function(noDelay)
{
    if(!noDelay)
    {
        if(updateBoardLayers.timer) clearTimeout(updateBoardLayers.timer);
        updateBoardLayers.timer = setTimeout(() => updateBoardLayers(true), 500);
        return;
    }
    const canvasID = zui.BoardEditor.getBoard().id;
    $.get($.createLink('board', 'ajaxUpdateLayer', 'canvasID=' + canvasID));
};

$(document).on('click', function(event)
{
    if(!$('body').hasClass('show-history')) return;
    if($(event.target).closest('#boardHistory,.history-toggle,.modal').length) return;
    toggleBoardHistory(false);
});

if(window.parent !== window && window.parent.name === 'app-board')
{
    document.addEventListener('click', function(event) {
        if (event.target.classList.contains('story-link'))
        {
            href = $(event.target).data('href');
            event.preventDefault();
            parent.zui.Modal.open({
                size: 'lg',
                url: href,
                onHidden: () => { updateBoardLayers(); }
            });
        }
    });

    $(window.document).off('locate.zt').on('locate.zt', (_, load) =>
    {
        if(load === 'modal')
        {
            parent.zui.Modal.hide();
            updateBoardLayers();
        }
        if(load) return updateBoardLayers();
        parent.$(parent.document).trigger('locate.zt', load);
    });
}

$('body').toggleClass('is-windows', navigator.platform.indexOf('Win') > -1);

$(document).on('blur', '.estimate-input', function()
{
    const canvasID = $('#boardHistory').data('canvasID');
    const storyID  = $(this).data('storyId');
    const type     = $(this).data('type');
    let value      = parseFloat($(this).val() ? $(this).val() : 0);
    if(value < 0) value = 0;

    $.ajaxSubmit({
        url: $.createLink('story', 'ajaxUpdateField', `storyID=${storyID}&objectType=${type}`),
        data: {
            field: 'estimate',
            value,
            canvasID
        },
        onSuccess: function()
        {
            updateBoardLayers();
        },
        onFail: (error) => {
            if(error?.message) zui.Modal.alert({message: error?.message.estimate});
        }
    });
});

$(document).on('keydown', '.estimate-input', function(event)
{
    /* 当按下回车键时，触发输入框的 blur 事件。 */
    /* When the enter key is pressed, the blur event of the input box is triggered. */
    if(event.key === 'Enter' || event.keyCode === 13 || event.which === 13) $(this)[0].blur();
});

const storyDropdownMap = new Map();
window.showPriList = function($target, storyID)
{
    if(storyDropdownMap.has($target)) return;

    const key     = $($target).attr('key');
    const type    = $($target).data('type');
    const priList = $('#boardHistory').data('priList');
    priList.forEach(pri => {
        pri['data-story'] = storyID;
        pri['data-type'] = type;
    });

    const dropdown = new zui.Dropdown($(`.story-pri[key=${key}]`), {
        items: priList,
        className: 'menu-pri-story'
    });

    storyDropdownMap.set($target, dropdown);
}

$(document).on('click', '.menu-pri-story .item-pri', function()
{
    const canvasID = $('#boardHistory').data('canvasID');
    const {pri, story, type} = $(this).data();
    $.ajaxSubmit({
        url: $.createLink('story', 'ajaxUpdateField', `storyID=${story}&objectType=${type}`),
        data: {
            field: 'pri',
            value: pri,
            canvasID
        },
        onSuccess: function()
        {
            updateBoardLayers();
        },
        onFail: (error) => {
            if(error?.message) zui.Modal.alert({message: error?.message.pri});
        }
    });
});

/**
 * 转化文字卡片。
 * Convert text cards.
 *
 * @param  string key
 * @param  array  textNotes
 * @access public
 * @return void
 */
window.onConvertObject = function(key, textNotes)
{
    if(textNotes.length == 1)
    {
        let cardName = textNotes[0].text ? textNotes[0].text.trim() : '';
        let   params = `productID=0&branch=0&moduleID=0&storyID=0&objectID=0&bugID=0&planID=0&todoID=0&extra=fromType=board,fromID=${canvasID}`;
        if(key == 'demand') params = `poolID=0&demandID=0&extra=fromType=board,fromID=${canvasID},boardID=${canvasID}`;
        if(key == 'epic')   params = `productID=0&branch=0&moduleID=0&storyID=0&objectID=0&bugID=0&planID=0&todoID=0&extra=fromType=board,fromID=${canvasID}`;
        if(params)
        {
            parent.zui.Modal.open(
                {
                    type: 'ajax',
                    request: {
                        url: $.createLink(key ? key : 'story', 'create', params),
                        method: 'post',
                        data: {storyName: cardName, subID: textNotes[0].id}
                    },
                    size: 'lg'
                }
            );
        }
    }
    if(textNotes.length > 1)
    {
        let storyNames = [];
        let subIDs = [];
        textNotes = textNotes.filter((item) => {return (!item.data || item.data.deleted != '0')});
        textNotes.forEach((item) => {
            storyNames.push(item.text.trim());
            subIDs.push(item.id);
        });
        parent.zui.Modal.open(
            {
                type: 'ajax',
                request: {
                    url: $.createLink('story', 'ajaxSelectProduct', `boardID=${canvasID}&storyType=${key ? key : 'projectstory'}`),
                    method: 'post',
                    data: {storyName: JSON.stringify(storyNames), subID: JSON.stringify(subIDs)}
                },
                size: 'sm'
            }
        );
    }
}
